﻿using Blahblah.FlowerStory.Server.Data;
using Blahblah.FlowerStory.Server.Data.Model;
using Microsoft.AspNetCore.Mvc;

namespace Blahblah.FlowerStory.Server.Controller;

/// <summary>
/// 事件相关服务
/// </summary>
[ApiController]
[Consumes("application/json")]
[Produces("application/json")]
[Route("api/event")]
public class EventController : BaseController
{
    /// <inheritdoc/>
    public EventController(FlowerDatabase database, ILogger<BaseController>? logger = null) : base(database, logger)
    {
    }

    /// <summary>
    /// 获取用户相关所有符合条件的事件
    /// </summary>
    /// <remarks>
    /// 请求示例：
    /// 
    ///     GET /api/event/query
    ///     Authorization: authorization id
    /// 
    /// 参数：
    /// 
    ///     cid: int?
    ///     key: string?
    ///     from: long?
    ///     to: long?
    ///     cfrom: decimal?
    ///     cto: decimal?
    /// 
    /// </remarks>
    /// <param name="eventId">事件类型 id</param>
    /// <param name="key">查询关键字</param>
    /// <param name="from">起始日期</param>
    /// <param name="to">结束日期</param>
    /// <returns>会话有效则返回符合条件的花草集</returns>
    /// <response code="200">返回符合条件的花草集</response>
    /// <response code="401">未找到登录会话或已过期</response>
    /// <response code="403">用户已禁用</response>
    /// <response code="404">未找到关联用户</response>
    [Route("query")]
    [ProducesResponseType(StatusCodes.Status200OK)]
    [ProducesResponseType(StatusCodes.Status401Unauthorized)]
    [ProducesResponseType(StatusCodes.Status403Forbidden)]
    [ProducesResponseType(StatusCodes.Status404NotFound)]
    [HttpGet]
    public ActionResult<RecordItem[]> GetRecords(
        [FromQuery(Name = "eid")] int? eventId,
        [FromQuery] string? key,
        [FromQuery] long? from,
        [FromQuery] long? to)
    {
        var (result, user) = CheckPermission();
        if (result != null)
        {
            return result;
        }
        if (user == null)
        {
            return NotFound();
        }

        SaveDatabase();

        var records = database.Records.Where(r => r.OwnerId == user.Id);
        if (eventId != null)
        {
            records = records.Where(r => r.EventId == eventId);
        }
        if (key != null)
        {
            records = records.Where(r =>
                r.ByUserName != null && r.ByUserName.ToLower().Contains(key.ToLower())
                // TODO: notes
                );
        }
        if (from != null)
        {
            records = records.Where(r => r.DateUnixTime >= from);
        }
        if (to != null)
        {
            records = records.Where(r => r.DateUnixTime <= to);
        }

        return Ok(records.ToArray());
    }
}
